@@ -10,16 +10,20 @@ SUPPORT_EXTENDED_SECURITY = True
 # Set to True if you want to enable SMB2 protocol.
 SUPPORT_SMB2 = True
 
-# Supported dialects
-DIALECTS = [ ]
-for i, ( name, dialect ) in enumerate([ ( 'NT_LAN_MANAGER_DIALECT', b'NT LM 0.12' ), ]):
-    DIALECTS.append(dialect)
-    globals()[name] = i
+# Set to True if you want to enable SMB2.1 and above protocol.
+SUPPORT_SMB2x = True
 
-DIALECTS2 = [ ]
-for i, ( name, dialect ) in enumerate([ ( 'SMB2_DIALECT', b'SMB 2.002' ) ]):
-    DIALECTS2.append(dialect)
-    globals()[name] = i + len(DIALECTS)
+# Supported dialects
+NT_LAN_MANAGER_DIALECT = 0  # 'NT LM 0.12' is always the first element in the dialect list and must always be included (MS-SMB 2.2.4.5.1)
+
+# Return the list of support SMB dialects based on the SUPPORT_x constants
+def init_dialects_list():
+    dialects = [ b'NT LM 0.12' ]
+    if SUPPORT_SMB2:
+        dialects.append(b'SMB 2.002')
+    if SUPPORT_SMB2x:
+        dialects.append(b'SMB 2.???')
+    return dialects
 
 
 class UnsupportedFeature(Exception):
@@ -107,8 +111,9 @@ class SMBMessage:
     log = logging.getLogger('SMB.SMBMessage')
     protocol = 1
 
-    def __init__(self, payload = None):
+    def __init__(self, conn, payload = None):
         self.reset()
+        self.conn = conn
         if payload:
             self.payload = payload
             self.payload.initMessage(self)
@@ -143,6 +148,10 @@ class SMBMessage:
         self.data = b''
         self.payload = None
 
+    @property
+    def isAsync(self):
+        return bool(self.flags & SMB2_FLAGS_ASYNC_COMMAND)
+
     @property
     def isReply(self):
         return bool(self.flags & SMB_FLAGS_REPLY)
@@ -261,7 +270,7 @@ class Payload:
         # rewritten to check for OEM/Unicode strings which will be tedious. Fortunately, almost all tested CIFS services
         # support SMB_FLAGS2_UNICODE by default.
         assert message.payload == self
-        message.flags =  SMB_FLAGS_CASE_INSENSITIVE
+        message.flags =  SMB_FLAGS_CASE_INSENSITIVE | SMB_FLAGS_CANONICALIZED_PATHS
         message.flags2 = SMB_FLAGS2_UNICODE | SMB_FLAGS2_NT_STATUS | SMB_FLAGS2_IS_LONG_NAME | SMB_FLAGS2_LONG_NAMES
 
         if SUPPORT_EXTENDED_SECURITY:
@@ -289,10 +298,7 @@ class ComNegotiateRequest(Payload):
     def prepare(self, message):
         assert message.payload == self
         message.parameters_data = b''
-        if SUPPORT_SMB2:
-            message.data = b''.join([b'\x02'+s+b'\x00' for s in DIALECTS + DIALECTS2])
-        else:
-            message.data = b''.join([b'\x02'+s+b'\x00' for s in DIALECTS])
+        message.data = b''.join(map(lambda s: b'\x02'+s+b'\x00', init_dialects_list()))
 
 
 class ComNegotiateResponse(Payload):
@@ -366,11 +372,11 @@ class ComNegotiateResponse(Payload):
                 if data_len >= self.challenge_length:
                     self.challenge = message.data[:self.challenge_length]
 
-                    s = ''
+                    s = b''
                     offset = self.challenge_length
                     while offset < data_len:
                         _s = message.data[offset:offset+2]
-                        if _s == '\0\0':
+                        if _s == b'\0\0':
                             self.domain = s.decode('UTF-16LE')
                             break
                         else:
@@ -412,7 +418,7 @@ class ComSessionSetupAndxRequest__WithSecurityExtension(Payload):
 
         message.flags2 |= SMB_FLAGS2_UNICODE
 
-        cap = CAP_UNICODE | CAP_STATUS32 | CAP_EXTENDED_SECURITY
+        cap = CAP_UNICODE | CAP_STATUS32 | CAP_EXTENDED_SECURITY | CAP_NT_SMBS
 
         message.parameters_data = \
             self.DEFAULT_ANDX_PARAM_HEADER + \
@@ -465,20 +471,20 @@ class ComSessionSetupAndxRequest__NoSecurityExtension(Payload):
 
         message.data = self.password
         if (est_offset + len(message.data)) % 2 != 0 and message.flags2 & SMB_FLAGS2_UNICODE:
-            message.data = message.data + '\0'
+            message.data = message.data + b'\0'
 
         if message.flags2 & SMB_FLAGS2_UNICODE:
-            message.data = message.data + self.username.encode('UTF-16LE') + '\0'
+            message.data = message.data + self.username.encode('UTF-16LE') + b'\0'
         else:
-            message.data = message.data + str(self.username) + '\0'
+            message.data = message.data + str(self.username).encode('UTF-8') + b'\0'
 
         if (est_offset + len(message.data)) % 2 != 0 and message.flags2 & SMB_FLAGS2_UNICODE:
-            message.data = message.data + '\0'
+            message.data = message.data + b'\0'
 
         if message.flags2 & SMB_FLAGS2_UNICODE:
-            message.data = message.data + self.domain.encode('UTF-16LE') + '\0\0' + 'pysmb'.encode('UTF-16LE') + '\0\0'
+            message.data = message.data + self.domain.encode('UTF-16LE') + b'\0\0' + 'pysmb'.encode('UTF-16LE') + b'\0\0'
         else:
-            message.data = message.data + self.domain + '\0pysmb\0'
+            message.data = message.data + self.domain.encode('UTF-8') + b'\0pysmb\0'
 
 
 class ComSessionSetupAndxResponse(Payload):
@@ -557,7 +563,7 @@ class ComTreeConnectAndxRequest(Payload):
             struct.pack(self.PAYLOAD_STRUCT_FORMAT,
                         0x08 | \
                             ((message.hasExtendedSecurity and 0x0004) or 0x00) | \
-                            ((message.tid and 0x0001) or 0x00),  # Disconnect tid, if message.tid must be non-zero
+                            ((message.tid and message.tid != 0xffff and 0x0001) or 0x00),  # Disconnect tid, if message.tid must be non-zero
                         password_len)
 
         padding = b''
@@ -1279,7 +1285,7 @@ class ComEchoRequest(Payload):
     - [MS-CIFS]: 2.2.4.39.1
     """
 
-    def __init__(self, echo_data = '', echo_count = 1):
+    def __init__(self, echo_data = b'', echo_count = 1):
         self.echo_count = echo_count
         self.echo_data = echo_data
 
